using System;
using System.Collections.Generic;
using System.Drawing;
using PaintDotNet; // resides in PaintDotNet.Core.dll
using PaintDotNet.Effects; // resides in PaintDotNet.Effects.dll

namespace MyNamespace
{
    public class MyEffect : Effect
    {
        public static string StaticName
        {
            get
            {
                return "My Effect's Name";
            }
        }

        public static Bitmap StaticIcon
        {
            get
            {
                return new Bitmap(typeof(MyEffect), "MyEffectIcon.png");
                // remember to set the icon's build process to "Embedded Resource"
            }
        }

        public MyEffect()
        : base(MyEffect.StaticName, MyEffect.StaticIcon, EffectFlags.Configurable)
        {
        }

        public override EffectConfigDialog CreateConfigDialog()
        {
            return new MyConfigDialog();
        }

        public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
        {
            MyConfigToken cd = (MyConfigToken)parameters;

            PdnRegion selectionRegion = EnvironmentParameters.GetSelection(srcArgs.Bounds);

            for (int i = startIndex; i < startIndex + length; i++)
            {
                Rectangle rect = rois[i];

                for (int y = rect.Top; y < rect.Bottom; y++)
                {
                    for (int x = rect.Left; x < rect.Right; x++)
                    {
                        ColorBgra c = srcArgs.Surface[x,y];
                        byte r = c.R;
                        byte g = c.G;
                        byte b = c.B;
                        if(cd.InvertColor)
                        {
                            r = (byte)(255-r);
                            g = (byte)(255-g);
                            b = (byte)(255-b);
                        }
                        dstArgs.Surface[x, y] = ColorBgra.FromBgr(b, g, r);
                    }
                }
            }
        }
    }

    public class MyConfigToken
    : EffectConfigToken
    {
        public bool InvertColor ;

        public MyConfigToken()
        : base()
        {
            InvertColor = false;
        }

        protected MyConfigToken(MyConfigToken copyMe)
        : base(copyMe)
        {
            this.InvertColor = copyMe.InvertColor;
        }

        public override object Clone()
        {
            return new MyConfigToken(this);
        }
    }
}